@@ -3,8 +3,6 @@ package Pugs::Emitter::Rule::Perl5::Ratchet;
 # p6-rule perl5 emitter for ":ratchet" (non-backtracking)
 # see: RuleInline.pl, RuleInline-more.pl for a program prototype
 
-# XXX - cleanup unused nodes
-
 use strict;
 use warnings;
 use Data::Dumper;
@@ -18,13 +16,15 @@ our $capture_to_array;
 our $count = 1000 + int(rand(1000));
 sub id { 'I' . ($count++) }
 
-# XXX - reuse this sub in metasyntax()
 sub call_subrule {
     my ( $subrule, $tab, @param ) = @_;
-    $subrule = "\$grammar->" . $subrule unless $subrule =~ / :: | \. | -> /x;
+    $subrule = "\$grammar->" . $subrule 
+        unless $subrule =~ / :: | \. | -> /x;
     $subrule =~ s/\./->/;   # XXX - source filter
     return 
-        "$tab     $subrule( \$s, { p => \$pos, args => {" . join(", ",@param) . "} }, undef )";
+"$tab     $subrule( \$s, { p => \$pos, args => {" .
+             join(", ",@param) . 
+         "} }, undef )";
 }
 
 sub quote_constant {
@@ -46,13 +46,9 @@ sub call_constant {
         unless length($_[0]);
     my $const = quote_constant( $_[0] );
     my $len = length( eval $const );
-    #print "Const: [$_[0]] $const $len \n";
-    
-    # return "$_[1] ( substr( \$s, \$pos$direction$direction, 1 ) eq $const )"
-    #    if $len == 1;
-    
+    #print "Const: [$_[0]] $const $len \n";    
     return
-    "$_[1] ( ( substr( \$s, \$pos, $len ) eq $const ) 
+"$_[1] ( ( substr( \$s, \$pos, $len ) eq $const ) 
 $_[1]     ? ( \$pos $direction= $len or 1 )
 $_[1]     : 0
 $_[1] )";
@@ -62,7 +58,7 @@ sub call_perl5 {
     my $const = $_[0];
     #print "CONST: $const - $direction \n";
     return
-    "$_[1] ( ( substr( \$s, \$pos ) =~ m/^($const)/s )  
+"$_[1] ( ( substr( \$s, \$pos ) =~ m/^($const)/s )  
 $_[1]     ? ( \$pos $direction= length( \$1 ) or 1 )
 $_[1]     : 0
 $_[1] )";
@@ -77,41 +73,43 @@ sub emit {
     local $capture_to_array = 0;
     #print "rule: ", Dumper( $ast );
     return 
-        "sub {\n" . 
-        "  my \$grammar = \$_[0];\n" .
-        "  my \$s = \$_[1];\n" .
-        "  no warnings 'substr', 'uninitialized', 'syntax';\n" .
-        "  my \%pad;\n" .
+        "do { my \$rule; \$rule = sub {
+  my \$grammar = \$_[0];
+  my \$s = \$_[1];
+  no warnings 'substr', 'uninitialized', 'syntax';
+  my \%pad;\n" .
         #"  my \$pos;\n" .
         #"  print \"match arg_list = \$_[1]\n\";\n" .
         #"  print 'match ', Dumper(\\\@_);\n" .
         #"  print \"match arg_list = \@{[\%{\$_[1]} ]}\n\" if defined \$_[1];\n" .
-        #"  \$pos = 0 unless defined \$pos;   # TODO - .*? \$match \n" .
         #"  print \"match pos = \$pos\n\";\n" .
-        "  my \$m;\n" .
-
-        #"  for my \$pos ( defined \$_[3]{p} ? \$_[3]{p} : ( 0 .. length( \$s ) - 1 ) ) {\n" .
-        "  for my \$pos ( defined \$_[3]{p} ? \$_[3]{p} : ( 0 .. length( \$s ) ) ) {\n" .
-
-        "    my \%index;\n" . 
-        "    my \@match;\n" .
-        "    my \%named;\n" .
-        #"  my \$from = \$pos;\n" .
-        "    my \$bool = 1;\n" .
-        "    \$named{KEY} = \$_[3]{KEY} if exists \$_[3]{KEY};\n" .
-        "    \$m = Pugs::Runtime::Match->new( { \n" .
-        "      str => \\\$s, from => \\(0+\$pos), to => \\(\$pos), \n" .
-        "      bool => \\\$bool, match => \\\@match, named => \\\%named, capture => undef, \n" .
-        "    } );\n" .
-        "    \$bool = 0 unless\n" .
+"  my \$m;
+  for my \$pos ( defined \$_[3]{p} ? \$_[3]{p} : ( 0 .. length( \$s ) ) ) {
+    my \%index; 
+    my \@match;
+    my \%named;
+    my \$bool = 1;
+    \$named{KEY} = \$_[3]{KEY} if exists \$_[3]{KEY};
+    \$m = Pugs::Runtime::Match->new( { 
+      str => \\\$s, from => \\(0+\$pos), to => \\(\$pos), 
+      bool => \\\$bool, match => \\\@match, named => \\\%named, capture => undef, 
+    } );
+    {
+      \$bool = 0 unless
+" .
         #"      do { TAILCALL: ;\n" .
-        emit_rule( $ast, '   ' ) . ";\n" .
-        #"      }\n" .
-        "    last if \$m;\n" .
-        "  }\n" .  # /for
-        "  \$::_V6_MATCH_ = \$m; \n" .
-        "  return \$m;\n" .
-        "}\n";
+        emit_rule( $ast, '    ' ) . ";
+    }
+    if ( \$bool ) {
+      \$::_V6_PRIOR_ = \$rule;
+      #print \"rule = \$::_V6_PRIOR_ \\n\";
+      last;
+    }
+  } # /for
+  \$::_V6_MATCH_ = \$m;
+  return \$m;
+} }
+";
 }
 
 sub emit_rule {
@@ -122,7 +120,6 @@ sub emit_rule {
     #print "NODE ", Dumper($n);
     my ($k) = keys %$n;
     my $v = $$n{$k};
-    #my ( $k, $v ) = each %$n;
     # XXX - use real references
     no strict 'refs';
     #print "NODE ", Dumper($k), ", ", Dumper($v);
@@ -248,63 +245,6 @@ sub code {
 }        
 sub dot {
     "$_[1] ( substr( \$s, \$pos$direction$direction, 1 ) ne '' )"
-    #if ( $direction eq '+' ) {
-    #    "$_[1] ( \$pos < length( \$s ) ? ++\$pos || 1 : 0 )"
-    #}
-    #else {
-    #    "$_[1] ( \$pos >= 0 ? --\$pos || 1 : 0 )"
-    #}
-}
-
-sub preprocess_hash {
-    # TODO - move to Runtime/
-    my ( $h, $key ) = @_;
-    # returns AST depending on $h
-    if ( ref( $h->{$key} ) eq 'CODE') {
-        return sub { 
-            my ( $str, $grammar, $args ) = @_;
-            #print "data: ", Dumper( \@_ );
-            my $ret = $h->{$key}->( @_ ); 
-            #print "ret: ", Dumper( $ret );
-            
-            return $ret 
-                if ref( $ret ) eq 'Pugs::Runtime::Match';
-            
-            Pugs::Runtime::Match->new( { 
-                bool => \1, 
-                str =>  \$str,
-                from => \( 0 + ( $args->{p} || 0 ) ),
-                to =>   \( 0 + ( $args->{p} || 0 ) ),
-            } ) }
-    } 
-    if ( ref( $h->{$key} ) =~ /Pugs::Compiler::/ ) {
-        return sub { $h->{$key}->match( @_ ) };
-    }
-    # fail is number != 1 
-    if ( $h->{$key} =~ /^(\d+)$/ ) {
-        return sub { 
-            my ( $str, $grammar, $args ) = @_;
-            Pugs::Runtime::Match->new( { 
-                bool => \0, 
-                str =>  \$str,
-                from => \( 0 + ( $args->{p} || 0 ) ),
-                to =>   \( 0 + ( $args->{p} || 0 ) ),
-            } ) } unless $1 == 1;
-        return sub { 
-            my ( $str, $grammar, $args ) = @_;
-            Pugs::Runtime::Match->new( { 
-                bool => \1, 
-                str =>  \$str,
-                from => \( 0 + ( $args->{p} || 0 ) ),
-                to =>   \( 0 + ( $args->{p} || 0 ) ),
-            } ) };
-    }
-    # subrule
-    #print "compile: ",$h->{$key}, "\n";
-    my $r = Pugs::Compiler::Token->compile( $h->{$key} );
-    $h->{$key} = $r;
-    return sub { $r->match( @_ ) };
-    # return sub { warn "uncompiled subrule: $h->{$key} - not implemented " };
 }
 
 sub variable {
@@ -336,7 +276,7 @@ sub variable {
     $value = join('', eval $name) if $name =~ /^\@/;
     if ( $name =~ /^%/ ) {
         my $id = '$' . id();
-        my $preprocess_hash = 'Pugs::Emitter::Rule::Perl5::Ratchet::preprocess_hash';
+        my $preprocess_hash = 'Pugs::Runtime::Regex::preprocess_hash';
         my $code = "
           do {
             our $id;
@@ -351,11 +291,6 @@ sub variable {
                 my \%sizes = map { length(\$_) => 1 } keys \%\$hash;
                 ${id}_sizes = [ sort { \$b <=> \$a } keys \%sizes ];
                 " . #print \"sizes: \@${id}_sizes\\n\";
-                #$id = {
-                #        map  { \$_ =>
-                #               Pugs::Emitter::Rule::Perl5::Ratchet::preprocess_hash( \$hash, \$_ ) }
-                #        keys \%\$hash
-                #};
                 "$id = \$hash;
             }
             " . #print 'keys: ',Dumper( $id );
@@ -508,8 +443,36 @@ $_[1]     \$match[ $capture_count ] = Pugs::Runtime::Match->new( \$hash );"
 $_[1]     \$bool;
 $_[1] }";
 }        
+
+sub capture_as_result {
+    my $program = $_[0];
+
+    $capture_count++;
+    {
+        local $capture_count = -1;
+        local $capture_to_array = 0;
+        $program = emit_rule( $program, $_[1].'      ' )
+            if ref( $program );
+    }
+    return "$_[1] do{ 
+$_[1]     my \$hash = do {
+$_[1]       my \$bool = 1;
+$_[1]       my \$from = \$pos;
+$_[1]       my \@match;
+$_[1]       my \%named;
+$_[1]       \$bool = 0 unless
+" .             $program . ";
+$_[1]       { str => \\\$s, from => \\\$from, match => \\\@match, named => \\\%named, bool => \\\$bool, to => \\(0+\$pos), capture => undef }
+$_[1]     };
+$_[1]     my \$bool = \${\$hash->{'bool'}};
+$_[1]     \$m->data->{capture} = \\( \"\" . Pugs::Runtime::Match->new( \$hash ) );
+$_[1]     \$bool;
+$_[1] }";
+}        
 sub named_capture {
     my $name    = $_[0]{ident};
+    $name = $name->{match_variable} if ref($name) eq 'HASH';
+    $name =~ s/^[\$\@\%]//;  # TODO - change semantics as needed
     my $program = $_[0]{rule};
     #print "name [$name]\n";
     
@@ -526,8 +489,10 @@ sub named_capture {
         $param_list = '' unless defined $param_list;
         my @param = split( ',', $param_list );
         return "$_[1] do { 
+                my \$prior = \$::_V6_PRIOR_; 
                 my \$match = \n" . 
                     call_subrule( $subrule, $_[1]."        ", @param ) . ";
+                \$::_V6_PRIOR_ = \$prior; 
                 if ( \$match ) {" .
                     ( $capture_to_array 
                     ? " push \@{\$named{'$name'}}, \$match;" 
@@ -670,8 +635,18 @@ sub colon {
         if $str eq '$';
     return "$_[1] ( \$pos == 0 ) \n" 
         if $str eq '^';
+        
+    return "$_[1] ( \$pos >= length( \$s ) || substr( \$s, \$pos ) =~ /^\\n/s ) \n" 
+        if $str eq '$$';
+    return "$_[1] ( \$pos == 0 || substr( \$s, 0, \$pos ) =~ /\\n\$/s ) \n" 
+        if $str eq '^^';
+
     die "'$str' not implemented";
 }
+sub modifier {
+    my $str = $_[0];
+    die "modifier '$str' not implemented";
+}
 sub constant {
     call_constant( @_ );
 }
@@ -679,21 +654,11 @@ sub constant {
 use vars qw( %char_class );
 BEGIN {
     %char_class = map { $_ => 1 } qw( 
-alpha
-alnum
-ascii
-blank
-cntrl
-digit
-graph
-lower
-print
-punct
-space
-upper
-word
-xdigit
-);
+        alpha alnum ascii blank
+        cntrl digit graph lower
+        print punct space upper
+        word  xdigit
+    );
 }
 
 sub metasyntax {
@@ -796,40 +761,21 @@ sub metasyntax {
         if ( exists $char_class{$cmd} ) {
             # XXX - inlined char classes are not inheritable, but this should be ok
             return call_perl5( "[[:$cmd:]]", $_[1] );
-#                "$_[1] ( ( substr( \$s, \$pos, 1 ) =~ /[[:$cmd:]]/ ) 
-# $_[1]     ? do { $direction$direction\$pos; 1 }
-# $_[1]     : 0
-# $_[1] )";
         }
+        my @param; # TODO
+        my $subrule = $cmd;
         return
-            "$_[1] do { my \$match =\n" .
-            call_subrule( $cmd, $_[1] . "          " ) . ";\n" .
-            "$_[1]      my \$bool = (!\$match != 1);\n" .
-            "$_[1]      \$pos = \$match->to if \$bool;\n" .
-            "$_[1]      \$match;\n" .
-            "$_[1] }";
-    }
-    if ( $prefix eq '!' ) {   # negated_subrule / code assertion 
-        $cmd = substr( $cmd, 1 );
-        if ( $cmd =~ /^{/ ) {
-            warn "code assertion not implemented";
-            return;
-        }
-        return 
-            "$_[1] ... negate( '$_[0]', \n" .
-            call_subrule( $_[0], $_[1]."  " ) .
-            "$_[1] )\n";
-    }
-    if ( $cmd eq '.' ) {
-            warn "<$cmd> not implemented";
-            return;
+"$_[1] do { 
+$_[1]      my \$prior = \$::_V6_PRIOR_; 
+$_[1]      my \$match = \n" . 
+               call_subrule( $subrule, $_[1]."        ", @param ) . ";
+$_[1]      \$::_V6_PRIOR_ = \$prior; 
+$_[1]      my \$bool = (!\$match != 1);
+$_[1]      \$pos = \$match->to if \$bool;
+$_[1]      \$match;
+$_[1] }";
     }
     if ( $prefix =~ /[_[:alnum:]]/ ) {  
-        # "before" and "after" are handled in a separate rule
-        #if ( $cmd eq 'redo' ) {
-        #    # tailcall - this is an unauthorized extension
-        #    "$_[1]      goto TAILCALL;\n";
-        #}
         if ( $cmd eq 'cut' ) {
             warn "<$cmd> not implemented";
             return;
@@ -838,27 +784,18 @@ sub metasyntax {
             warn "<$cmd> not implemented";
             return;
         }
-        if ( $cmd eq 'prior' ) {
-            warn "<$cmd> not implemented";
-            return;
-        }
         if ( $cmd eq 'null' ) {
             return "$_[1] 1 # null\n"
         }
-
-        # XXX - disabled, because this doesn't capture
-        # if ( exists $char_class{$cmd} ) {
-        #    # XXX - inlined char classes are not inheritable, but this should be ok
-        #    return
-        #        "$_[1] ( ( substr( \$s, \$pos, 1 ) =~ /[[:$cmd:]]/ ) 
-        # $_[1]     ? do { $direction$direction\$pos; 1 }
-        # $_[1]     : 0
-        # $_[1] )";
-        # }
-
-        # capturing subrule
         # <subrule ( param, param ) >
         my ( $subrule, $param_list ) = split( /[\(\)]/, $cmd );
+        $param_list ||= '';
+
+        if ( $subrule eq 'at' ) {
+            $param_list ||= 0;   # XXX compile-time only
+            return "$_[1] ( \$pos == $param_list )\n"
+        }
+
         return named_capture(
             { 
                 ident => $subrule, 
@@ -867,6 +804,12 @@ sub metasyntax {
             $_[1],    
         );
     }
+    if ( $prefix eq '.' ) {  
+        my ( $method, $param_list ) = split( /[\(\)]/, $cmd );
+        $method =~ s/^\.//;
+        $param_list ||= '';
+        return " ( \$s->$method( $param_list ) ? 1 : 0 ) ";
+    }
     die "<$cmd> not implemented";
 }
 
